/*
 * flexi-cgi is a simple flexitime cgi recorder
 * Copyright (C) 2000 V.R.Sanders
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact V Sanders email
 * vince@kyllikki.fluff.org
 *
 * The main web site for this program is
 * http://www.kyllikki.fluff.org/software
 *
 * flexi-cgi uses cgi-util LGPL library by Bill Kendrick (see cgi-util directory)
 */

/* standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>
#include <strings.h>
#include <time.h>

/* library headers */
#include "cgi-util.h"

/* program headers */
#include "flexi-cgi.h"

int send_error(const char * err_txt)
{
    printf("<head>\n<title>Error :%s</title>\n</head><body>\n<h1>Error :%s</h1>\n</body>\n",err_txt,err_txt);
    return 0;
}

int flexi_user_list(const char * docroot)
{
    char * user_dir_name;
    DIR * user_dir;
    struct dirent * user_dir_entry;
    int first=1;

    user_dir_name=malloc(strlen(docroot)+strlen(FLEXI_CGI_ROOT)+2);

    strcpy(user_dir_name,docroot);
    strcat(user_dir_name,"/"FLEXI_CGI_ROOT);

    user_dir=opendir(user_dir_name);

    if (user_dir==NULL)
    {
	send_error("Unable to acess flexi sheets directory");
	return -1;
    }

    printf("<h2>Select User</h2>");
    printf("<form action=\"/cgi-bin/flexi-cgi\" method=\"post\">");

    printf("<select name=\"user\">");

    while((user_dir_entry=readdir(user_dir)))
    {
	/* skip hidden files */
	if (user_dir_entry->d_name[0]=='.')
	    continue;
	if(first)
	    printf("<option selected>%s",
		   user_dir_entry->d_name);
	else
	    printf("<option>%s",
		   user_dir_entry->d_name);
    }

    printf("</select>");
    printf("<hr><input type=\"submit\" name=\"action\" value=\"Select\">\n");
    printf("</form>");

    closedir(user_dir);
    free(user_dir_name);
    return 0;
}

int flexi_check_passwd(const char * docroot,const char * user,const char * password)
{
    char * paswd_file_name;
    FILE * paswd_file;
    char * paswd_read;
    int result=0;
    int loop=0;

    if (user==NULL)
	return 0;

    paswd_file_name=malloc(strlen(docroot)+strlen(FLEXI_CGI_ROOT)+strlen(user)+20);

    strcpy(paswd_file_name,docroot);
    strcat(paswd_file_name,"/"FLEXI_CGI_ROOT"/");
    strcat(paswd_file_name,user);
    strcat(paswd_file_name,"/.paswd");

    paswd_file=fopen(paswd_file_name,"r");

    /* no file? no passwd allow acess - dodgy */
    if (paswd_file==NULL)
	return 1;

    if (password!=NULL)
    {

	paswd_read=malloc(1024);
	paswd_read[fread(paswd_read,1,1024,paswd_file)]=0;

	while(paswd_read[loop])
	{
	    if(paswd_read[loop]<32)
		paswd_read[loop]=0;
	    loop++;
	}


	if(strcmp(paswd_read,password)==0)
	    result=1;

	free(paswd_read);
    }

    fclose(paswd_file);

    free(paswd_file_name);


    return result;
}

int flexi_prompt_passwd(const char * docroot,const char * user)
{
    if (user==NULL)
    {
	/* no user specified so list available ones */
     	flexi_user_list(docroot);
	return 0;
    }

    printf("<h2>Password Required</h2>\n");
    printf("<form action=\"/cgi-bin/flexi-cgi\" method=\"post\">\n");

    printf("<input type=\"hidden\" name=\"user\" value=\"%s\">\n",user);

    printf("Password required for user %s",user);

    printf("<p>Password <input type=\"password\" name=\"password\">\n");

    printf("<hr><input type=\"submit\" name=\"action\" value=\"Select\">\n");
    printf("</form>\n");

    return 0;
}

int get_week_number()
{
    struct tm* tm_timenow;
    char week_num[128];
    time_t timenow;

    timenow=time(NULL);
    tm_timenow=gmtime(&timenow);
    strftime(week_num,128,"%W",tm_timenow);
    return(atoi(week_num));

}

/* find the weekday number 0 based from monday */
int get_week_day_number()
{
    struct tm* tm_timenow;
    time_t timenow;
    int day;

    timenow=time(NULL);
    tm_timenow=gmtime(&timenow);
    day=tm_timenow->tm_wday;
    day--;
    if(day==-1)
	day=6;
    return(day);
}

int do_table_cell(const time_t * toshow,const char * dayname,const char * cell_name)
{
    struct tm *tm_time;
    tm_time=gmtime(toshow);

    printf("<td><input type=\"text\" name=\"%s_h_%s\" value=\"%.2d\" maxlength=2 size=3>",dayname,cell_name, tm_time->tm_hour);

    printf(":<input type=\"text\" name=\"%s_m_%s\" value=\"%.2d\" maxlength=2 size=3>\n",dayname,cell_name,tm_time->tm_min);
    return 0;
}


int do_table_cell_total(const time_t toshow)
{
    int hours,mins;
    hours=(toshow)/3600;
    mins=((toshow)-(hours*3600))/60;
    if((hours<0) || (mins<0))
    {
	/* deal with negative time printing nicely */
	hours=0-hours;
	mins=0-mins;
    printf("<td><h4>-%.2d:%.2d</h4>", hours,mins);
    }
    else
    printf("<td><h4>%.2d:%.2d</h4>", hours,mins);
    return 0;
}

int do_table_day(const fx_day *day)
{
    switch(day->daytype)
    {
    case DAYTYPE_BANKHOL:
	printf("<tr><th>%s is a Bank Holiday!\n",
	       day->dayname);

	do_table_cell(&day->inout[0],day->dayname,"inout_0");
	do_table_cell(&day->inout[1],day->dayname,"inout_1");
	do_table_cell(&day->inout[2],day->dayname,"inout_2");
	do_table_cell(&day->inout[3],day->dayname,"inout_3");
	do_table_cell_total(day->day_total);

	printf("</tr>\n");
	break;

    case DAYTYPE_HOLIDAY:
	printf("<tr><th>%s has been taken as holiday\n</tr>",
	       day->dayname);
	break;

    case DAYTYPE_SICK:
	printf("<tr><th>%s was a sick day\n</tr>",
	       day->dayname);
	break;

    case DAYTYPE_CONFERENCE:
	printf("<tr><th>%s was a conference day\n</tr>",
	       day->dayname);
	break;

    default:
	printf("<tr><th>%s\n",day->dayname);

	do_table_cell(&day->inout[0],day->dayname,"inout_0");
	do_table_cell(&day->inout[1],day->dayname,"inout_1");
	do_table_cell(&day->inout[2],day->dayname,"inout_2");
	do_table_cell(&day->inout[3],day->dayname,"inout_3");
	do_table_cell_total(day->day_total);

	printf("</tr>\n");
    }

    return 0;
}

FILE * open_sheet(const char * docroot,const char * user,int week_num,char* mode)
{
    FILE * sheet_file;
    char * sheet_file_name;

    if (user==NULL)
	return NULL;

    sheet_file_name=malloc(strlen(docroot)+strlen(FLEXI_CGI_ROOT)+strlen(user)+20);

    sprintf(sheet_file_name,"%s/%s/%s/week-%d",docroot,FLEXI_CGI_ROOT,user,week_num);

    sheet_file=fopen(sheet_file_name,mode);

    if (sheet_file==NULL)
    {
	free(sheet_file_name);
	return NULL;
    }

    free(sheet_file_name);

    return sheet_file;
}

int calc_normal_hours(fx_week * calcweek)
{
    int loop;
    int total=0;

    for(loop=0;loop<7;loop++)
    {
	if(calcweek->days[loop].daytype==DAYTYPE_WORKING)
	    total+=calcweek->work_day_length;
    }
    return total;
}



int read_week_control(const char * docroot,fx_week * calcweek)
{
    FILE * ctrl_file;

    ctrl_file=open_sheet(docroot,".control",calcweek->weekno,"r");

    if (ctrl_file==NULL)
    {
	/* use defaults if cannot open control file */
	calcweek->work_day_length=27000; /* 7.5 hours */
	calcweek->days[5].daytype_default=DAYTYPE_WEEKEND;
	calcweek->days[6].daytype_default=DAYTYPE_WEEKEND;
    }
    else
    {
	fscanf(ctrl_file,
	       "%ld %d %d %d %d %d %d %d",
	       &calcweek->work_day_length,
	       &calcweek->days[0].daytype_default,
	       &calcweek->days[1].daytype_default,
	       &calcweek->days[2].daytype_default,
	       &calcweek->days[3].daytype_default,
	       &calcweek->days[4].daytype_default,
	       &calcweek->days[5].daytype_default,
	       &calcweek->days[6].daytype_default);

    fclose(ctrl_file);

    }

    return 0;
}

fx_week* initweek(const char * docroot,int weeknumber)
{
    fx_week * toset;
    int loop;

    toset=malloc(sizeof(fx_week));
    if (toset==NULL)
	return NULL;

    memset(toset,0,sizeof(fx_week));

    toset->days[0].dayname = "Monday";
    toset->days[1].dayname = "Tuesday";
    toset->days[2].dayname = "Wednesday";
    toset->days[3].dayname = "Thursday";
    toset->days[4].dayname = "Friday";

    toset->days[5].dayname = "Saturday";
    toset->days[6].dayname = "Sunday";

    toset->weekno=weeknumber;
    toset->dayno=get_week_day_number();

    /* read master control file for week and calc normal hours */
    read_week_control(docroot,toset);

    for (loop=0;loop<7;loop++)
    {
	toset->days[loop].daytype=toset->days[loop].daytype_default;
    }

    toset->week_length=calc_normal_hours(toset);

    return toset;
}

int calc_worked_hours(fx_week * updateweek)
{
    int loop,loop2;
    int week_tot_time=0,tot_time=0;
    fx_day * day;

    /* get vars of the form <day>_[hm]_inout_<num> */
    for (loop=0;loop<7;loop++)
    {
	day=&updateweek->days[loop];
	tot_time=0;

	/* claculate day total time */
	for (loop2=0;loop2<4;loop2+=2)
	{
	    tot_time+=((day->inout[loop2+1]) - (day->inout[loop2]));
	}

	day->day_total=tot_time;

	week_tot_time+=updateweek->days[loop].day_total;
    }

    updateweek->week_total=week_tot_time;

    return week_tot_time;
}

int update_week_days(fx_day * day)
{
    char * cgi_entry_name;
    int loop;
    int type;
    int hour,min;
    int tot_time=0;

    cgi_entry_name=malloc(1024);

    for (loop=0;loop<4;loop++)
    {
	sprintf(cgi_entry_name,"%s_h_inout_%d",day->dayname,loop);
	hour=cgi_getentryint(cgi_entry_name);
	sprintf(cgi_entry_name,"%s_m_inout_%d",day->dayname,loop);
	min=cgi_getentryint(cgi_entry_name);
	if(!((hour==0) && (min==0)))
	    day->inout[loop]=((hour*60*60)+(min*60));
    }

    /* check for daytype changes - only if global type is normal day*/
    sprintf(cgi_entry_name,"%s_daytype",day->dayname);
    type=cgi_getentryint(cgi_entry_name);
    if (type!=0)
    {
	if (type>DAYTYPE_GLOBALEND)
	    day->daytype = type;
	else
	    day->daytype = day->daytype_default;
    }

    free(cgi_entry_name);

    /* ensure user input never goes back in time! */
    for (loop=1;loop<4;loop++)
    {
	if(day->inout[loop-1] > day->inout[loop])
	    day->inout[loop]=day->inout[loop-1];
    }

    /* claculate day total time */
    for (loop=0;loop<4;loop+=2)
    {
	tot_time+=((day->inout[loop+1]) - (day->inout[loop]));
    }

    day->day_total=tot_time;

    return 0;

}


int update_week(fx_week * updateweek)
{
    int loop;
    int week_tot_time=0;

    /* get vars of the form <day>_[hm]_inout_<num> */
    for (loop=0;loop<7;loop++)
    {
	update_week_days(&updateweek->days[loop]);
	week_tot_time+=updateweek->days[loop].day_total;
    }
    updateweek->week_total=week_tot_time;

    updateweek->week_length=calc_normal_hours(updateweek);

    return 0;
}



int save_week(fx_week * saveweek,const char * docroot,const char * user)
{
    FILE * sheet_file;
    int loop;

    if (user==NULL)
	return 1;

    sheet_file=open_sheet(docroot,user,saveweek->weekno,"w+");
    if (sheet_file==NULL)
    {
	printf("<!-- unable to write to sheet file -->\n");
	return 0;
    }

    for(loop=0;loop<7;loop++)
    {
	fprintf(sheet_file,"%ld %ld %ld %ld\n",
		saveweek->days[loop].inout[0],
		saveweek->days[loop].inout[1],
		saveweek->days[loop].inout[2],
		saveweek->days[loop].inout[3]);
    }

    fprintf(sheet_file,"%d %d %d %d %d %d %d\n",
	    saveweek->days[0].daytype,
	    saveweek->days[1].daytype,
	    saveweek->days[2].daytype,
	    saveweek->days[3].daytype,
	    saveweek->days[4].daytype,
	    saveweek->days[5].daytype,
	    saveweek->days[6].daytype);

    fclose(sheet_file);
    return 0;
}

fx_week* new_week(const char * docroot,const char * user,int week_num)
{
    fx_week *newweek;

    if (user==NULL)
	return NULL;

    newweek=initweek(docroot,week_num);

    save_week(newweek,docroot,user);

    return(newweek);
}

fx_week* load_week(const char * docroot,const char * user,int week_num)
{
    FILE * sheet_file;
    fx_week *newweek;
    int loop,scanned;
    long daytype[7];

    if (user==NULL)
	return NULL;

    newweek=initweek(docroot,week_num);

    sheet_file=open_sheet(docroot,user,week_num,"r");

    if (sheet_file==NULL)
    {
	free(newweek);
	return NULL;
    }

    for(loop=0;loop<7;loop++)
    {
	scanned=fscanf(sheet_file,"%ld %ld %ld %ld\n",
		       &newweek->days[loop].inout[0],
		       &newweek->days[loop].inout[1],
		       &newweek->days[loop].inout[2],
		       &newweek->days[loop].inout[3]);

	/* check we have the right number of inputs and clear any in error */
	if (scanned!=4)
	{
	    if (scanned<0)
		scanned=0;
	    while(scanned<4)
	    {
		newweek->days[loop].inout[scanned]=0;
		scanned++;
	    }
	}

    }

    /* get the daytypes (for hols etc) check they are valid before use */
    scanned=fscanf(sheet_file,"%ld %ld %ld %ld %ld %ld %ld\n",
		&daytype[0],
		&daytype[1],
		&daytype[2],
		&daytype[3],
		&daytype[4],
		&daytype[5],
		&daytype[6]);

    /* check we have the right number of inputs and clear any in error */
    if (scanned!=7)
    {
	if (scanned<0)
	    scanned=0;
	while(scanned<7)
	{
	    daytype[scanned]=0;
	    scanned++;
	}
    }

    for(loop=0;loop<7;loop++)
    {
	if(daytype[loop]>DAYTYPE_GLOBALEND)
	    newweek->days[loop].daytype=daytype[loop];
    }

    newweek->week_length=calc_normal_hours(newweek);

    fclose(sheet_file);

    return(newweek);

}

int list_weeks(fx_week * thisweek,char * docroot,const char *user,const char * password)
{
    /* list previous weeks to select from */
    char * user_dir_name;
    DIR * user_dir;
    struct dirent * user_dir_entry;

    user_dir_name=malloc(strlen(docroot)+strlen(FLEXI_CGI_ROOT)+strlen(user)+3);

    strcpy(user_dir_name,docroot);
    strcat(user_dir_name,"/"FLEXI_CGI_ROOT"/");
    strcat(user_dir_name,user);

    user_dir=opendir(user_dir_name);

    if (user_dir==NULL)
    {
	send_error("Unable to access users sheet directory");
	return -1;
    }

    printf("<h2>Available Weeks</h2>");
    printf("<form action=\"/cgi-bin/flexi-cgi\" method=\"post\">");

    printf("<input type=\"hidden\" name=\"user\" value=\"%s\">\n",user);

    if(password!=NULL)
	printf("<input type=\"hidden\" name=\"password\" value=\"%s\">\n",password);

    printf("<select name=\"week\">");

    while((user_dir_entry=readdir(user_dir)))
    {
	/* skip hidden files */
	if (user_dir_entry->d_name[0]=='.')
	    continue;
	if(atoi(&user_dir_entry->d_name[5]) == thisweek->weekno )
	    printf("<option selected>");
	else
	    printf("<option>");

	printf("%s",&user_dir_entry->d_name[5]);
    }

    printf("</select>");
    printf("<hr><input type=\"submit\" name=\"action\" value=\"Select\">\n");
    printf("</form>");

    closedir(user_dir);
    free(user_dir_name);
    return 0;
}

int list_summaries(fx_week * listweek,const char * docroot,const char * user,const char * password)
{
    fx_week * sum_week;
    int loop;
    int num_weeks=0;
    int last_week=0;
    int last_4=0;

    long total_worked=0;
    long total_required=0;
    long total_4_worked=0;
    long total_4_required=0;

    /*    int num_holidays=0;
    int num_sickdays=0;
    int num_conferencedays=0;*/

    loop=last_week=get_week_number()-1;/* dont include this week */

    for (;loop>=0;loop--)
    {
	sum_week=load_week(docroot,user,loop);

	if (sum_week!=NULL)
	{
	    last_4++;
	    calc_worked_hours(sum_week);
	    num_weeks++;
	    total_worked+=sum_week->week_total;
	    total_required+=sum_week->week_length;
	    if (last_4==4)
	    {
		total_4_worked=total_worked;
		total_4_required=total_required;
	    }

	    free(sum_week);
	}
    }

    printf("<head><title>Summary Sheet For %s</title></head><body>",user);
    printf("<h2>Summary Sheet For %s</h2>",user);

    printf("<form action=\"/cgi-bin/flexi-cgi\" method=\"post\">\n");

    printf("<input type=hidden name=\"user\" value=\"%s\">\n",user);

    if(password!=NULL)
	printf("<input type=hidden name=\"password\" value=\"%s\">\n",password);
    printf("<input type=hidden name=\"week\" value=\"%d\">\n",listweek->weekno);

    printf("<p>Complete records exist for a total of %d weeks",num_weeks);
    printf("<p>Totals for weeks %d to %d",(last_week-num_weeks+1),last_week);
    printf("<p>Total required hours were %ld",(total_required/3600));
    printf("<p>Total worked hours were %ld",(total_worked/3600));
    printf("<p>Excess of %ld hours",((total_worked-total_required)/3600));
    printf("<p>Totals for last four weeks");
    printf("<p>Total required hours were %ld",(total_4_required/3600));
    printf("<p>Total worked hours were %ld",(total_4_worked/3600));
    printf("<p>Excess of %ld hours",((total_4_worked-total_4_required)/3600));

    printf("<hr><input type=submit name=\"action\" value=\"Done\">\n");
    printf("</form>\n");

    return 0;
}

int list_leave(fx_week * listweek,const char * docroot,const char * user,const char * password)
{
    fx_day * day;
    int loop;
    int typeloop;

    printf("<head><title>Leave Sheet For %s</title></head><body>",user);

    printf("<h2>Leave Sheet For %s</h2>",user);

    printf("<h3>Today is %s in week number %d</h3>",listweek->days[listweek->dayno].dayname,listweek->weekno);

    printf("<form action=\"/cgi-bin/flexi-cgi\" method=\"post\">\n");

    printf("<input type=hidden name=\"user\" value=\"%s\">\n",user);

    if(password!=NULL)
	printf("<input type=hidden name=\"password\" value=\"%s\">\n",password);
    printf("<input type=hidden name=\"week\" value=\"%d\">\n",listweek->weekno);

    printf("<table cellspacing=6>\n");
    printf("<tr><th><th>Normal<th>Sick<th>Holiday<th>Conference</tr>\n");

    for (loop=0;loop<7;loop++)
    {
	day = &listweek->days[loop];
	printf("<tr><td>%s",day->dayname);

	for(typeloop=DAYTYPE_GLOBALEND;typeloop<=DAYTYPE_USEREND;typeloop++)
	{
	    printf("<td><input type=radio");
	    if (typeloop>DAYTYPE_GLOBALEND)
	    {
		if(day->daytype==typeloop)
		    printf(" checked");
	    }
	    else
	    {
		if(day->daytype<=DAYTYPE_GLOBALEND)
		    printf(" checked");
	    }
		printf(" name=\"%s_daytype\" value=\"%.3d\">",
		       day->dayname,
		       typeloop );


	}
	printf("</tr>\n");

    }
    printf("</table>");

    printf("<hr><input type=submit name=\"action\" value=\"Update\">\n");
    printf("</form>\n");
    return 0;
}

int week_input(fx_week * thisweek,const char * docroot,const char * user,const char * password)
{
    int loop;

    printf("<head><title>Flexi Time Sheet For %s</title></head><body>",user);

    printf("<h2>Flexi Time Sheet For %s</h2>",user);

    printf("<h3>Today is %s in week number %d</h3>",thisweek->days[thisweek->dayno].dayname,thisweek->weekno);

    printf("<form action=\"/cgi-bin/flexi-cgi\" method=\"post\">\n");

    printf("<input type=\"hidden\" name=\"user\" value=\"%s\">\n",user);

    if(password!=NULL)
	printf("<input type=\"hidden\" name=\"password\" value=\"%s\">\n",password);
    printf("<input type=\"hidden\" name=\"week\" value=\"%d\">\n",thisweek->weekno);

    printf("<table cellspacing=6>");
    printf("<tr><th><th colspan=5>Times</tr>\n");
    printf("<tr><th><th>In<th>Out<th>In<th>Out<th>Total</tr>\n");

    for (loop=0;loop<7;loop++)
    {
	do_table_day(&thisweek->days[loop]);
    }

    printf("<tr><td><td><td><td><td><h4>Week Total</h4>");
    do_table_cell_total(thisweek->week_total);
    printf("</tr>\n");

    printf("<tr><td><td><td><td><td><h4>Normal hours</h4>");
    do_table_cell_total(thisweek->week_length);
    printf("</tr>\n");

    printf("<tr><td><td><td><td><td><h4>Total Excess</h4>");
    do_table_cell_total(thisweek->week_total-thisweek->week_length);
    printf("</tr>\n");

    printf("</table>");
    printf("<hr><input type=\"submit\" name=\"action\" value=\"Update\">\n");
    printf("<input type=\"submit\" name=\"action\" value=\"List\">\n");
    printf("<input type=\"submit\" name=\"action\" value=\"Leave\">\n");
    printf("<input type=\"submit\" name=\"action\" value=\"Summaries\">\n");
    printf("</form></body>\n");
    return 0;
}


int flexi_show_sheet(char * docroot,const char * user,const char * password,int weekno)
{
    fx_week *thisweek;
    const char * action;

    if ((user==NULL) || (strlen(user)==0))
    {
	printf("<H2>Error user was empty</h2>");
	return 1;
    }

    if (weekno==0)
	weekno=get_week_number();

    /* load week file into struct or create a new one as apropriate */
    thisweek=load_week(docroot,user,weekno);
    if (thisweek==NULL)
	thisweek=new_week(docroot,user,weekno);

    /* keep any changes made and calc totals */
    update_week(thisweek);

    /* save changes */
    save_week(thisweek,docroot,user);

    /* if the user updated the form ensure we save the updates */
    action=cgi_getentrystr("action");
    if ((action!=NULL) && (strcasecmp(action,"list"))==0)
    {
	list_weeks(thisweek,docroot,user,password);
	return 0;
    }
    else if ((action!=NULL) && (strcasecmp(action,"leave"))==0)
    {
	list_leave(thisweek,docroot,user,password);
	return 0;
    }
    else if ((action!=NULL) && (strcasecmp(action,"summaries"))==0)
    {
	list_summaries(thisweek,docroot,user,password);
	return 0;
    }
    else
	week_input(thisweek,docroot,user,password);

    return 0;
}


int main(int argc, char * argv[])
{
    int res;
    char * doc_root;

    /* Initialize the CGI and send out an HTTP header: */

    res = cgi_init();

    printf("Content-type: text/html\n\n");


    /* Was there an error initializing the CGI??? */

    if (res != CGIERR_NONE)
    {
	printf("Error # %d: %s<p>\n", res, cgi_strerror(res));
	exit(0);
    }

    doc_root=getenv("DOCUMENT_ROOT");
    if (doc_root == NULL)
    {
	send_error("Document Root Not Found");
	exit(0);
    }

    /* boilerplate top here */
    printf("<html>\n");


    if(flexi_check_passwd(doc_root,
			  cgi_getentrystr("user"),
			  cgi_getentrystr("password")))
    {
	flexi_show_sheet(doc_root,
			 cgi_getentrystr("user"),
			 cgi_getentrystr("password"),
			 cgi_getentryint("week"));
    }
    else
	flexi_prompt_passwd(doc_root,cgi_getentrystr("user"));


    /* boilerplate bottom here */
    printf("</html>\n");

    cgi_quit();

    return(0);
}
